#!/bin/bash
# Grabs the content of a cached ceph-disk list
eexit() {
  local error_str="$@"
  echo $error_str
  exit 1
}

readonly card=$1
readonly option="DISPLAY"
readonly loadcap=15
readonly oldage=604800
readonly midage=86400

readonly cache_file=/tmp/sas2ircu_display_${card}
readonly cmd="/usr/local/sbin/sas2ircu ${card} ${option}"

readonly PROGNAME=$(basename "$0")
readonly LOCKFILE_DIR=/tmp
readonly LOCK_FD=200

lock() {
  local prefix="${1}_${card}"
  local fd=${3:-$LOCK_FD}
  local lock_file=$LOCKFILE_DIR/$prefix.lock

  # Create lock File
  eval "exec $fd>$lock_file"

  # acquier the lock
  flock -n $fd && return 0 || return 1
}

high_load() {
  load=$(awk '{print $1}' /proc/loadavg | cut -d '.' -f 1)
  [ "$load" -lt "$1" ] && return 0
  return 1
}
older_than() {
  file=$1
  agelimit=$2
  age=$(($(date +%s) - $(stat -L --format %Y $file)))
  return $(( age  > agelimit ))
}

update_cache_file() {
  lock $PROGNAME $DIRECTORY || return
  $($cmd > $cache_file)
}

main() {
  run
}

can_update() {
  # Yes if it doesn't exist
  [ ! -e $cache_file ] && return 0
  # Yes if it's ancient
  older_than $cache_file $old_age
  [ "$?" -eq 0 ] && return 0
  # Yes if it's quite old and the load is low
  older_than $cache_file $midage
  old=$?
  high_load $loadcap
  too_high=$?
  [ "$old" -eq 0 ] && ! [ "$too_high" -eq 0 ] && return 0
  # No if it's fresh
  return 1
}

run() {
  can_update
  [ "$?" -eq 0 ] && update_cache_file
  cat $cache_file
}

main
